//Example for Seeed Studio's 3-Color E-Ink display
// Copyright (C) 2022 Raymond Allen
// MIT License
// Modified by RJA to work on Parallax Propeller II using FlexProp 5.9.6

//RJA: The original is here:  https://github.com/Seeed-Studio/Seeed_Elink_Shield
//RJA: This file is an adaptation of the Aruduino C++ file epd2in7b-demo.ino
//RJA: Overall, the main change is turning the C++ to plain C and adding P2 specific pin work in epdif.c

#include "platform.h"  //P2 settings and includes are here



/*************************************************************************
  Includes
*************************************************************************/

#include "Teensy-ICM-20948.h"



// InvenSense drivers and utils
#include "Icm20948.h"
#include "SensorTypes.h"
#include "Icm20948MPUFifoControl.h"

#include "Icm20948SelfTest.c"
#include "Icm20948Augmented.c"
#include "Icm20948LoadFirmware.c"
#include "Icm20948AuxCompassAkm.c"
#include "Icm20948AuxTransport.c"
#include "Icm20948DataBaseDriver.c"
#include "Icm20948DataConverter.c"
#include "Icm20948Dmp3Driver.c"
#include "Icm20948DataBaseControl.c"
#include "Icm20948Transport.c"
#include "Icm20948Setup.c"
#include "Icm20948MPUFifoControl.c"

/*************************************************************************
  SPI
*************************************************************************/
#include "SPI.c"
Spi SPI;

// Pin definition
#define SPI_CS      51
#define SPI_MISO    50
#define SPI_SCK     49
#define SPI_MOSI    48

#define LOW 0
#define HIGH 1

void DigitalWrite(int pin, int value);
int  DigitalRead(int pin);
void DelayMs(unsigned int delaytime);
void SpiTransfer(unsigned char data);

void DigitalWrite(int pin, int value) {
    //digitalWrite(pin, value);
    _pinw(pin, value);
}

int DigitalRead(int pin) {
    //return digitalRead(pin);
    return _pinr(pin);
}

void DelayMs(unsigned int delaytime) {
    //delay(delaytime);
    _waitms(delaytime);
}

void SpiWrite(unsigned char data) {

    //_pinl(SPI_CS);
    SPI.Write8(SPI_MOSI, SPI_SCK, data);
    //_pinh(SPI_CS);
}

unsigned char SpiRead() {
    //_pinl(SPI_CS);
    return SPI.Read8(SPI_MISO, SPI_SCK);
    //_pinh(SPI_CS);
}


/*************************************************************************
  Settings
*************************************************************************/

///* Animation Settings
TeensyICM20948Settings settings = {
  SPI_CS,                   // SPI chip select pin
  7000000,           // SPI clock speed in Hz, max speed is 7MHz
  1,                      // 0 = low power mode, 1 = high performance mode
  false,      // Enables gyroscope output
  false,  // Enables accelerometer output
  false,   // Enables magnetometer output
  true,      // Enables quaternion output
  1,       // Max frequency = 225, min frequency = 1
  1,   // Max frequency = 225, min frequency = 1
  1,    // Max frequency = 70, min frequency = 1
  225     // Max frequency = 225, min frequency = 50
};
//*/

/*
TeensyICM20948Settings settings =
{
  .cs_pin = 10,                  // SPI chip select pin
  .spi_speed = 7000000,          // SPI clock speed in Hz, max speed is 7MHz
  .mode = 1,                     // 0 = low power mode, 1 = high performance mode
  .enable_gyroscope = true,      // Enables gyroscope output
  .enable_accelerometer = true,  // Enables accelerometer output
  .enable_magnetometer = true,   // Enables magnetometer output
  .enable_quaternion = true,     // Enables quaternion output
  .gyroscope_frequency = 1,      // Max frequency = 225, min frequency = 1
  .accelerometer_frequency = 1,  // Max frequency = 225, min frequency = 1
  .magnetometer_frequency = 1,   // Max frequency = 70, min frequency = 1
  .quaternion_frequency = 50     // Max frequency = 225, min frequency = 50
};
*/



/*************************************************************************
  Variables
*************************************************************************/

int chipSelectPin = SPI_CS;// 10;
int spiSpeed = 7000000;

float gyro_x, gyro_y, gyro_z;
bool gyro_data_ready = false;

float accel_x, accel_y, accel_z;
bool accel_data_ready = false;

float mag_x, mag_y, mag_z;
bool mag_data_ready = false;

float quat_w, quat_x, quat_y, quat_z;
bool quat_data_ready = false;

/*************************************************************************
  HAL Functions for Arduino
*************************************************************************/

int idd_io_hal_read_reg(void* context, uint8_t reg, uint8_t* rbuffer, uint32_t rlen)
{
    (void)context;

    //SPI.beginTransaction(SPISettings(spiSpeed, MSBFIRST, SPI_MODE0));
    //SPI.transfer(0x00);
    //SPI.endTransaction();

    DigitalWrite(chipSelectPin, LOW);
    //SPI.beginTransaction(SPISettings(spiSpeed, MSBFIRST, SPI_MODE0));
    SpiWrite(((reg & 0x7F) | 0x80));
    for (uint32_t indi = 0; indi < rlen; indi++)
    {
        *(rbuffer + indi) = SpiRead();// SpiTransfer(0x00);
    }
    //SPI.endTransaction();
    DigitalWrite(chipSelectPin, HIGH);

    return 0;
}

int idd_io_hal_write_reg(void* context, uint8_t reg, const uint8_t* wbuffer, uint32_t wlen)
{
    (void)context;

    //SPI.beginTransaction(SPISettings(spiSpeed, MSBFIRST, SPI_MODE0));
    //SPI.transfer(0x00);
    //SPI.endTransaction();

    DigitalWrite(chipSelectPin, LOW);
    //SPI.beginTransaction(SPISettings(spiSpeed, MSBFIRST, SPI_MODE0));
    SpiWrite((reg & 0x7F) | 0x00);
    for (uint32_t indi = 0; indi < wlen; indi++)
    {
        SpiWrite(*(wbuffer + indi));
    }
    //SPI.endTransaction();
    DigitalWrite(chipSelectPin, HIGH);

    return 0;
}

/*************************************************************************
  Invensense Variables
*************************************************************************/

inv_icm20948_t icm_device;
int rc = 0;
static const uint8_t EXPECTED_WHOAMI[] = { 0xEA }; /* WHOAMI value for ICM20948 or derivative */
#define AK0991x_DEFAULT_I2C_ADDR  0x0C
#define THREE_AXES 3
static int unscaled_bias[THREE_AXES * 2];

static const float cfg_mounting_matrix[9] = {
  1.f, 0, 0,
  0, 1.f, 0,
  0, 0, 1.f
};

int32_t cfg_acc_fsr = 4; // Default = +/- 4g. Valid ranges: 2, 4, 8, 16
int32_t cfg_gyr_fsr = 2000; // Default = +/- 2000dps. Valid ranges: 250, 500, 1000, 2000

static const uint8_t dmp3_image[] = {
#include "icm20948_img.dmp3a.h"
};

/*************************************************************************
  Invensense Functions
*************************************************************************/

void check_rc(int rc, const char* msg_context)
{
    if (rc < 0) {
        printf("ICM20948 ERROR!\n");
        while (1);
    }
}

int load_dmp3(void)
{
    int rc = 0;
    rc = inv_icm20948_load(&icm_device, dmp3_image, sizeof(dmp3_image));
    return rc;
}

void inv_icm20948_sleep_us(int us)
{
    _waitus(us);
}

void inv_icm20948_sleep(int ms)
{
    _waitms(ms);
}

uint64_t inv_icm20948_get_time_us(void)
{
    return _getus();  //micros();
}

inv_bool_t interface_is_SPI(void)
{
    return true;
}

static void icm20948_apply_mounting_matrix(void)
{
    int ii;

    for (ii = 0; ii < INV_ICM20948_SENSOR_MAX; ii++) {
        inv_icm20948_set_matrix(&icm_device, cfg_mounting_matrix, ii);//RJA (inv_icm20948_sensor)ii);
    }
}

static void icm20948_set_fsr(void)
{
    inv_icm20948_set_fsr(&icm_device, INV_ICM20948_SENSOR_RAW_ACCELEROMETER, (const void*)&cfg_acc_fsr);
    inv_icm20948_set_fsr(&icm_device, INV_ICM20948_SENSOR_ACCELEROMETER, (const void*)&cfg_acc_fsr);
    inv_icm20948_set_fsr(&icm_device, INV_ICM20948_SENSOR_RAW_GYROSCOPE, (const void*)&cfg_gyr_fsr);
    inv_icm20948_set_fsr(&icm_device, INV_ICM20948_SENSOR_GYROSCOPE, (const void*)&cfg_gyr_fsr);
    inv_icm20948_set_fsr(&icm_device, INV_ICM20948_SENSOR_GYROSCOPE_UNCALIBRATED, (const void*)&cfg_gyr_fsr);
}



int icm20948_sensor_setup(void)
{
    int rc;
    uint8_t i, whoami = 0xff;

    inv_icm20948_soft_reset(&icm_device);

    // Get whoami number
    rc = inv_icm20948_get_whoami(&icm_device, &whoami);

    uint8_t EXPECTED_WHOAMI2[1];
    EXPECTED_WHOAMI2[0] = 0xEA;

    printf("\nwhoami=%d == %d? \n", whoami, EXPECTED_WHOAMI2[0]);

    // Check if WHOAMI value corresponds to any value from EXPECTED_WHOAMI array
    for (i = 0; i < sizeof(EXPECTED_WHOAMI2) / sizeof(EXPECTED_WHOAMI2[0]); ++i) {
        if (whoami == EXPECTED_WHOAMI2[i]) {
            break;
        }
    }

    if (i == sizeof(EXPECTED_WHOAMI2) / sizeof(EXPECTED_WHOAMI2[0])) {
        printf("Bad WHOAMI value = 0x\n",whoami);
        //Serial.println(whoami, HEX);
        return rc;
    }

    printf("Who=%d\n", whoami);

    // Setup accel and gyro mounting matrix and associated angle for current board
    inv_icm20948_init_matrix(&icm_device);

    // set default power mode
    rc = inv_icm20948_initialize(&icm_device, dmp3_image, sizeof(dmp3_image));
    if (rc != 0) {
        printf("Icm20948 Initialization failed.\n");
        return rc;
    }

    // Configure and initialize the ICM20948 for normal use

    // Initialize auxiliary sensors
    inv_icm20948_register_aux_compass(&icm_device, INV_ICM20948_COMPASS_ID_AK09916, AK0991x_DEFAULT_I2C_ADDR);

    rc = inv_icm20948_initialize_auxiliary(&icm_device);
    if (rc == -1) {
        printf("Compass not detected...\n");
    }


    icm20948_apply_mounting_matrix();

    icm20948_set_fsr();

    // re-initialize base state structure
    inv_icm20948_init_structure(&icm_device);

    return 0;
}

static uint8_t icm20948_get_grv_accuracy(void)
{
    uint8_t accel_accuracy;
    uint8_t gyro_accuracy;

    accel_accuracy = (uint8_t)inv_icm20948_get_accel_accuracy();
    gyro_accuracy = (uint8_t)inv_icm20948_get_gyro_accuracy();
    return (min(accel_accuracy, gyro_accuracy));
}

static uint8_t convert_to_generic_ids[INV_ICM20948_SENSOR_MAX] = {
  INV_SENSOR_TYPE_ACCELEROMETER,
  INV_SENSOR_TYPE_GYROSCOPE,
  INV_SENSOR_TYPE_RAW_ACCELEROMETER,
  INV_SENSOR_TYPE_RAW_GYROSCOPE,
  INV_SENSOR_TYPE_UNCAL_MAGNETOMETER,
  INV_SENSOR_TYPE_UNCAL_GYROSCOPE,
  INV_SENSOR_TYPE_BAC,
  INV_SENSOR_TYPE_STEP_DETECTOR,
  INV_SENSOR_TYPE_STEP_COUNTER,
  INV_SENSOR_TYPE_GAME_ROTATION_VECTOR,
  INV_SENSOR_TYPE_ROTATION_VECTOR,
  INV_SENSOR_TYPE_GEOMAG_ROTATION_VECTOR,
  INV_SENSOR_TYPE_MAGNETOMETER,
  INV_SENSOR_TYPE_SMD,
  INV_SENSOR_TYPE_PICK_UP_GESTURE,
  INV_SENSOR_TYPE_TILT_DETECTOR,
  INV_SENSOR_TYPE_GRAVITY,
  INV_SENSOR_TYPE_LINEAR_ACCELERATION,
  INV_SENSOR_TYPE_ORIENTATION,
  INV_SENSOR_TYPE_B2S
};

void build_sensor_event_data(void* context, enum inv_icm20948_sensor sensortype, uint64_t timestamp, const void* data, const void* arg)
{
    //*
    float raw_bias_data[6];
    inv_sensor_event_t event;
    (void)context;
    //printf("Sensor Type=%d\n", sensortype);
    uint8_t sensor_id = convert_to_generic_ids[sensortype];
    //printf("ID=%d, sensor=%d\n", sensor_id, sensortype);

    //for (int i = 0; i < 20; i++)
    //{
    //    printf("ID[%d] = %d\n", i, convert_to_generic_ids[i]);
    //}

    memset((void*)&event, 0, sizeof(event));
    event.sensor = sensor_id;
    event.timestamp = timestamp;

    //printf("Sensor ID=%d\n", sensor_id);

    //sensor_id = INV_SENSOR_TYPE_GAME_ROTATION_VECTOR;
    //*/

    /*
    float raw_bias_data[6];
    inv_sensor_event_t event;
    (void)context;
    uint8_t sensor_id = convert_to_generic_ids[sensortype];

    memset((void*)&event, 0, sizeof(event));
    event.sensor = sensor_id;
    event.timestamp = timestamp;
    */


    switch (sensor_id)
    {
    case INV_SENSOR_TYPE_UNCAL_GYROSCOPE:
        memcpy(raw_bias_data, data, sizeof(raw_bias_data));
        memcpy(event.data.gyr.vect, &raw_bias_data[0], sizeof(event.data.gyr.vect));
        memcpy(event.data.gyr.bias, &raw_bias_data[3], sizeof(event.data.gyr.bias));
        memcpy(&(event.data.gyr.accuracy_flag), arg, sizeof(event.data.gyr.accuracy_flag));
        break;
    case INV_SENSOR_TYPE_UNCAL_MAGNETOMETER:
        memcpy(raw_bias_data, data, sizeof(raw_bias_data));
        memcpy(event.data.mag.vect, &raw_bias_data[0], sizeof(event.data.mag.vect));
        memcpy(event.data.mag.bias, &raw_bias_data[3], sizeof(event.data.mag.bias));
        memcpy(&(event.data.gyr.accuracy_flag), arg, sizeof(event.data.gyr.accuracy_flag));
        break;
    case INV_SENSOR_TYPE_GYROSCOPE:
        memcpy(event.data.gyr.vect, data, sizeof(event.data.gyr.vect));
        memcpy(&(event.data.gyr.accuracy_flag), arg, sizeof(event.data.gyr.accuracy_flag));

        // WE WANT THIS
        gyro_x = event.data.gyr.vect[0];
        gyro_y = event.data.gyr.vect[1];
        gyro_z = event.data.gyr.vect[2];
        gyro_data_ready = true;
        break;

    case INV_SENSOR_TYPE_GRAVITY:
        memcpy(event.data.acc.vect, data, sizeof(event.data.acc.vect));
        event.data.acc.accuracy_flag = inv_icm20948_get_accel_accuracy();
        break;
    case INV_SENSOR_TYPE_LINEAR_ACCELERATION:
    case INV_SENSOR_TYPE_ACCELEROMETER:
        memcpy(event.data.acc.vect, data, sizeof(event.data.acc.vect));
        memcpy(&(event.data.acc.accuracy_flag), arg, sizeof(event.data.acc.accuracy_flag));

        // WE WANT THIS
        accel_x = event.data.acc.vect[0];
        accel_y = event.data.acc.vect[1];
        accel_z = event.data.acc.vect[2];
        accel_data_ready = true;
        break;

    case INV_SENSOR_TYPE_MAGNETOMETER:
        memcpy(event.data.mag.vect, data, sizeof(event.data.mag.vect));
        memcpy(&(event.data.mag.accuracy_flag), arg, sizeof(event.data.mag.accuracy_flag));

        // WE WANT THIS
        mag_x = event.data.mag.vect[0];
        mag_y = event.data.mag.vect[1];
        mag_z = event.data.mag.vect[2];
        //printf("Mag x=%f,y=%f, z=%f\n", mag_x, mag_y, mag_z);
        mag_data_ready = true;
        
        break;

    case INV_SENSOR_TYPE_GEOMAG_ROTATION_VECTOR:
    case INV_SENSOR_TYPE_ROTATION_VECTOR:
        memcpy(&(event.data.quaternion.accuracy), arg, sizeof(event.data.quaternion.accuracy));
        memcpy(event.data.quaternion.quat, data, sizeof(event.data.quaternion.quat));
        break;
    case INV_SENSOR_TYPE_GAME_ROTATION_VECTOR:
        memcpy(event.data.quaternion.quat, data, sizeof(event.data.quaternion.quat));
        event.data.quaternion.accuracy_flag = icm20948_get_grv_accuracy();

        // WE WANT THIS
        quat_w = event.data.quaternion.quat[0];
        quat_x = event.data.quaternion.quat[1];
        quat_y = event.data.quaternion.quat[2];
        quat_z = event.data.quaternion.quat[3];
        quat_data_ready = true;
        break;

    case INV_SENSOR_TYPE_BAC:
        memcpy(&(event.data.bac.event), data, sizeof(event.data.bac.event));
        break;
    case INV_SENSOR_TYPE_PICK_UP_GESTURE:
    case INV_SENSOR_TYPE_TILT_DETECTOR:
    case INV_SENSOR_TYPE_STEP_DETECTOR:
    case INV_SENSOR_TYPE_SMD:
        event.data.event = true;
        break;
    case INV_SENSOR_TYPE_B2S:
        event.data.event = true;
        memcpy(&(event.data.b2s.direction), data, sizeof(event.data.b2s.direction));
        break;
    case INV_SENSOR_TYPE_STEP_COUNTER:
        memcpy(&(event.data.step.count), data, sizeof(event.data.step.count));
        break;
    case INV_SENSOR_TYPE_ORIENTATION:
        //we just want to copy x,y,z from orientation data
        memcpy(&(event.data.orientation), data, 3 * sizeof(float));
        break;
    case INV_SENSOR_TYPE_RAW_ACCELEROMETER:
    case INV_SENSOR_TYPE_RAW_GYROSCOPE:
        memcpy(event.data.raw3d.vect, data, sizeof(event.data.raw3d.vect));
        break;
    default:
        return;
    }
}

static enum inv_icm20948_sensor idd_sensortype_conversion(int sensor)
{
    switch (sensor)
    {
    case INV_SENSOR_TYPE_RAW_ACCELEROMETER:
        return INV_ICM20948_SENSOR_RAW_ACCELEROMETER;
    case INV_SENSOR_TYPE_RAW_GYROSCOPE:
        return INV_ICM20948_SENSOR_RAW_GYROSCOPE;
    case INV_SENSOR_TYPE_ACCELEROMETER:
        return INV_ICM20948_SENSOR_ACCELEROMETER;
    case INV_SENSOR_TYPE_GYROSCOPE:
        return INV_ICM20948_SENSOR_GYROSCOPE;
    case INV_SENSOR_TYPE_UNCAL_MAGNETOMETER:
        return INV_ICM20948_SENSOR_MAGNETIC_FIELD_UNCALIBRATED;
    case INV_SENSOR_TYPE_UNCAL_GYROSCOPE:
        return INV_ICM20948_SENSOR_GYROSCOPE_UNCALIBRATED;
    case INV_SENSOR_TYPE_BAC:
        return INV_ICM20948_SENSOR_ACTIVITY_CLASSIFICATON;
    case INV_SENSOR_TYPE_STEP_DETECTOR:
        return INV_ICM20948_SENSOR_STEP_DETECTOR;
    case INV_SENSOR_TYPE_STEP_COUNTER:
        return INV_ICM20948_SENSOR_STEP_COUNTER;
    case INV_SENSOR_TYPE_GAME_ROTATION_VECTOR:
        return INV_ICM20948_SENSOR_GAME_ROTATION_VECTOR;
    case INV_SENSOR_TYPE_ROTATION_VECTOR:
        return INV_ICM20948_SENSOR_ROTATION_VECTOR;
    case INV_SENSOR_TYPE_GEOMAG_ROTATION_VECTOR:
        return INV_ICM20948_SENSOR_GEOMAGNETIC_ROTATION_VECTOR;
    case INV_SENSOR_TYPE_MAGNETOMETER:
        return INV_ICM20948_SENSOR_GEOMAGNETIC_FIELD;
    case INV_SENSOR_TYPE_SMD:
        return INV_ICM20948_SENSOR_WAKEUP_SIGNIFICANT_MOTION;
    case INV_SENSOR_TYPE_PICK_UP_GESTURE:
        return INV_ICM20948_SENSOR_FLIP_PICKUP;
    case INV_SENSOR_TYPE_TILT_DETECTOR:
        return INV_ICM20948_SENSOR_WAKEUP_TILT_DETECTOR;
    case INV_SENSOR_TYPE_GRAVITY:
        return INV_ICM20948_SENSOR_GRAVITY;
    case INV_SENSOR_TYPE_LINEAR_ACCELERATION:
        return INV_ICM20948_SENSOR_LINEAR_ACCELERATION;
    case INV_SENSOR_TYPE_ORIENTATION:
        return INV_ICM20948_SENSOR_ORIENTATION;
    case INV_SENSOR_TYPE_B2S:
        return INV_ICM20948_SENSOR_B2S;
    default:
        return INV_ICM20948_SENSOR_MAX;
    }
}

/*************************************************************************
  Class Functions
*************************************************************************/

void init() //TeensyICM20948Settings settings)
{
    printf("Initializing ICM-20948...\n");

    // Setup SPI
    chipSelectPin = SPI_CS;
    //spiSpeed = settings.spi_speed;
    //pinMode(chipSelectPin, OUTPUT);
    DigitalWrite(chipSelectPin, HIGH);
    //SPI.begin();

    

    SPI.Write8(SPI_MOSI, SPI_CS, 0);
    //SPI.beginTransaction(SPISettings(spiSpeed, MSBFIRST, SPI_MODE0));
    //SPI.transfer(0x00);
    //SPI.endTransaction();

    // Initialize icm20948 serif structure
    struct inv_icm20948_serif icm20948_serif;
    icm20948_serif.context = 0; // no need
    icm20948_serif.read_reg = idd_io_hal_read_reg;
    icm20948_serif.write_reg = idd_io_hal_write_reg;
    icm20948_serif.max_read = 1024 * 16; // maximum number of bytes allowed per serial read
    icm20948_serif.max_write = 1024 * 16; // maximum number of bytes allowed per serial write
    icm20948_serif.is_spi = interface_is_SPI();

    

    // Reset icm20948 driver states
    inv_icm20948_reset_states(&icm_device, &icm20948_serif);
    inv_icm20948_register_aux_compass(&icm_device, INV_ICM20948_COMPASS_ID_AK09916, AK0991x_DEFAULT_I2C_ADDR);


    short int_read_back;
    char buff[128];
    inv_icm20948_identify_interrupt(&icm_device, &int_read_back);
    //printf("Int=%d", int_read_back);
    //Serial.println(buff);

    // Setup the icm20948 device
    rc = icm20948_sensor_setup();

    if (icm_device.selftest_done && !icm_device.offset_done)
    {
        // If we've run self test and not already set the offset.
        inv_icm20948_set_offset(&icm_device, unscaled_bias);
        icm_device.offset_done = 1;
    }


    // Now that Icm20948 device is initialized, we can proceed with DMP image loading
    // This step is mandatory as DMP image is not stored in non volatile memory
    rc += load_dmp3(); 
    check_rc(rc, "Error sensor_setup/DMP loading.");

    // Set mode
    inv_icm20948_set_lowpower_or_highperformance(&icm_device, settings.mode);

    // Set frequency
    rc = inv_icm20948_set_sensor_period(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_GAME_ROTATION_VECTOR), 1000 / settings.quaternion_frequency);
    rc = inv_icm20948_set_sensor_period(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_GYROSCOPE), 1000 / settings.gyroscope_frequency);
    rc = inv_icm20948_set_sensor_period(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_ACCELEROMETER), 1000 / settings.accelerometer_frequency);
    rc = inv_icm20948_set_sensor_period(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_MAGNETOMETER), 1000 / settings.magnetometer_frequency);

    // Enable / disable
    rc = inv_icm20948_enable_sensor(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_GYROSCOPE), settings.enable_gyroscope);
    rc = inv_icm20948_enable_sensor(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_ACCELEROMETER), settings.enable_accelerometer);
    rc = inv_icm20948_enable_sensor(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_GAME_ROTATION_VECTOR), settings.enable_quaternion);
    rc = inv_icm20948_enable_sensor(&icm_device, idd_sensortype_conversion(INV_SENSOR_TYPE_MAGNETOMETER), settings.enable_magnetometer);

}

void task()
{
    inv_icm20948_poll_sensor(&icm_device, (void*)0, build_sensor_event_data);
}

bool gyroDataIsReady()
{
    return gyro_data_ready;
}

bool accelDataIsReady()
{
    return accel_data_ready;
}

bool magDataIsReady()
{
    return mag_data_ready;
}

bool quatDataIsReady()
{
    return quat_data_ready;
}

void readGyroData(float* x, float* y, float* z)
{
    *x = gyro_x;
    *y = gyro_y;
    *z = gyro_z;
    gyro_data_ready = false;
}

void readAccelData(float* x, float* y, float* z)
{
    *x = accel_x;
    *y = accel_y;
    *z = accel_z;
    accel_data_ready = false;
}

void readMagData(float* x, float* y, float* z)
{
    *x = mag_x;
    *y = mag_y;
    *z = mag_z;
    mag_data_ready = false;
}

void readQuatData(float* w, float* x, float* y, float* z)
{
    *w = quat_w;
    *x = quat_x;
    *y = quat_y;
    *z = quat_z;
    quat_data_ready = false;
}

///* Animation Loop
void loop()
{
    float quat_w, quat_x, quat_y, quat_z;
    char sensor_string_buff[128];




    // Must call this often in main loop -- updates the sensor values
    task();

    if (quatDataIsReady())
    {
        readQuatData(&quat_w, &quat_x, &quat_y, &quat_z);

        // Store data in a JSON string and send it over the serial port
        printf("{\"quat_w\":%f, \"quat_x\":%f, \"quat_y\":%f, \"quat_z\":%f}\r\n", quat_w, quat_x, quat_y, quat_z);
        
    }

}
//*/

/*
void loop()
{
    float gyro_x, gyro_y, gyro_z;
    float accel_x, accel_y, accel_z;
    float mag_x, mag_y, mag_z;
    float quat_w, quat_x, quat_y, quat_z;
    char sensor_string_buff[128];



    // Must call this often in main loop -- updates the sensor values
    task();

    if (gyroDataIsReady())
    {
        readGyroData(&gyro_x, &gyro_y, &gyro_z);
        //sprintf(sensor_string_buff, "Gyro (deg/s): [%f,%f,%f]", gyro_x, gyro_y, gyro_z);
        //Serial.println(sensor_string_buff);
        printf("Gyro (deg/s): [%f,%f,%f]\n", gyro_x, gyro_y, gyro_z);

    }

    if (accelDataIsReady())
    {
        readAccelData(&accel_x, &accel_y, &accel_z);
        //sprintf(sensor_string_buff, "Accel (g): [%f,%f,%f]", accel_x, accel_y, accel_z);
        //Serial.println(sensor_string_buff);
        printf("Accel (g): [%f,%f,%f]\n", accel_x, accel_y, accel_z);
    }

    if (magDataIsReady())
    {
        readMagData(&mag_x, &mag_y, &mag_z);
        //sprintf(sensor_string_buff, "Mag (uT): [%f,%f,%f]", mag_x, mag_y, mag_z);
        //Serial.println(sensor_string_buff);
        printf("Mag (uT): [%f,%f,%f]\n", mag_x, mag_y, mag_z);
    }

    if (quatDataIsReady())
    {
        readQuatData(&quat_w, &quat_x, &quat_y, &quat_z);
        //sprintf(sensor_string_buff, "Quat (deg): [%f,%f,%f,%f]", quat_w, quat_x, quat_y, quat_z);
        //Serial.println(sensor_string_buff);
        printf("Quat (deg): [%f,%f,%f,%f]\n", quat_w, quat_x, quat_y, quat_z);
    }
}
*/

int main() {
    //check clock freq, can be changed by enum in Platform.h
    int cf = _clockfreq();
    printf("clock freq= %d\n", cf);

    int iMountResult=-1;   
#if defined(USE_SD)
    iMountResult mount("/sd", _vfs_open_sdcard());
    chdir("/sd");
#elif defined(USE_HOST)
    iMountResult = mount("/host", _vfs_open_host());
    chdir("/host");
#else
    #warning no file system specified
#endif
    printf("File system mount returned %d.\n", iMountResult);
    

    SPI.Start(10, 0);

    printf("waiting for DMP...\n");
    sleep(5);


    init();

    //loop();

    printf("Done.\n");
    for (;;) 
    {
        loop();
        //_waitms(1000);
    }
}

